function Calibration=CalibrateLoadCell()

    answer=questdlg('Load existing calibration or perform new calibration?','Calibration','From File','New - Shunt','New - Mass','From File');
    switch(answer)
        case('From File')%load existing calibration
                [calibration_filename,calibration_path]=uigetfile('Select load cell calibration file','Calibration.mat','Calibration\*.mat');
                load(fullfile(calibration_path,calibration_filename),'Calibration');

                t_now=datetime;
                t_cal=Calibration.datetime;
                calibration_age=between(t_cal,t_now,'days');
                too_old=30;%days calibration is considered valid
                if(caldays(calibration_age)>too_old)
                    msgbox(['WARNING: Calibration over ',int2str(too_old),' days old. Please recalibrate.']);
                end
        case('New - Shunt')
            %53.6g measured via rigorous manual calibration w/ 150kOhm shunt: see ManualCalibration.xlsx and 
            %% http://www.pcb.com/contentstore/mktgcontent/techsupport/docs/ftq/TN-9-0202_Shunt_Calibration.pdf
            mass=[0 53.63997617 0];
            Cal_Volts=zeros(size(mass));
            
            %% Do the calibration
            %% Zero load
            uiwait(msgbox('Clear load cell then click OK.','Calibration'));
            Cal_Volts(1)=ReadLoadCellNTimes(1e2,'@AN[1]-@AN[2]');
            
            %% Shunt calibration
            uiwait(msgbox('Press and hold SHUNT button on amplifier, then click OK.','Calibration'));
            Cal_Volts(2)=ReadLoadCellNTimes(1e2,'@AN[1]-@AN[2]');
            
            %% Zero load
            uiwait(msgbox('Release SHUNT button, then click OK.','Calibration'));
            Cal_Volts(3)=ReadLoadCellNTimes(1e2,'@AN[1]-@AN[2]');
    
            Calibration=FindCalibrationCurve(mass,Cal_Volts,'SHUNT');
            msgbox('New shunt-based calibration complete.','Calibration');
        case('New - Mass')%do new calibration
    
            mass=[0 1 5 10 20];%grams
            
            CalWt={'Clear load cell then click OK.'};
            for(i=2:length(mass))
                if(mass(i)==0)
                    CalWt=[CalWt;'Clear load cell.'];
                elseif(mass(i)<1)%mg masses
                    unit_str=' mg';
                    unit_multiplier=1000;
                    CalWt=[CalWt;'Place ',int2str(mass(i)*unit_multiplier),unit_str,' calibration mass.'];
                else%gram masses
                    unit_str=' g';
                    unit_multiplier=1;
                    CalWt=[CalWt;'Place ',int2str(mass(i)*unit_multiplier),unit_str,' calibration mass.'];
                end
            end
    
            %% Do the calibration
            for(i=1:length(CalWt))
                button = questdlg(CalWt{i},'Calibration','OK','Halt','OK');
                switch(button)
                    case('Halt')%close the form
                        mass=mass(1:length(Cal_Volts));%truncate x to match the size of Cal_Volts
                        break;
                    case('OK')
                        Cal_Volts(i)=ReadLoadCellNTimes(2e2,'@AN[1]-@AN[2]'); %#ok<AGROW> 
                end
            end
            Calibration=FindCalibrationCurve(mass,Cal_Volts,'MASS');
            msgbox('New mass-based calibration complete.','Calibration');
    end

end

function Calibration=FindCalibrationCurve(mass,Cal_Volts,Cal_Type)

    ADCRange=2;%-10 to +10 VDC
    ADCDifferential=true;

    %% Analyze the calibration data
    y=Cal_Volts(:);%raw values
    Weight=mass/1000*9.80665;%gram-mass to Newtons
    X = [ones(length(Weight),1) Weight(:)];
    b = X\y;%least squares
    y_fit=b(1)+b(2)*Weight;%calibration line
    size(y),size(y_fit)
    Rsq=1-sum((y(:)-y_fit(:)).^2)/sum((y(:)-mean(y(:))).^2);
    
    %% Generate the calibration figure
    CalFig=figure;
    plot(Weight,Cal_Volts,'k.',Weight,b(1)+b(2)*Weight,'r-');
    set(gca,'FontSize',16);
    xlabel('Calibration Weight (N)');
    % ylabel('ADC Counts (-)');
    ylabel('Load Cell Voltage (V)');
    legend('Data','Calibration Curve');
    legend('boxoff');
    
    if(b(2)>=0)
        sign_of_sensitivity='+';
    else
        sign_of_sensitivity='-';
    end
    title(sprintf(['V[V]=',num2str(b(1)),'[V]',sign_of_sensitivity,num2str(abs(b(2))),'[V/N]*F[N] with R^2=',num2str(Rsq,'%11.4f')]));
    fname=['Calibration - ',Cal_Type,' - ',datestr(clock)];
    fname(strfind(fname,':'))='-';
        
    if(~isfolder('Calibration')),mkdir('Calibration');end
    saveas(CalFig,['Calibration\',fname,'.fig']);
    
    %% Do some error checking to make sure calibration is good
    if(max(Cal_Volts)>9.95)%+10V max ADC range
        msgbox('WARNING:  Sensor saturated! Inspect calibration curve and use with caution.');
    end
    if(Rsq<0.95)
        msgbox('WARNING:  R^2 value below 0.95 for calibration! Inspect calibration curve and use with caution.');
    end
    if(abs(b(2))>30 || abs(b(2))<10)
        msgbox('WARNING:  Sensitivity outside of expected range (30-50 V/N).  Inspect calibration curve and use with caution.');
    end
    Calibration.ADCRange=ADCRange;
    Calibration.Differential=ADCDifferential;
    Calibration.b=b;%coefficient vector
    Calibration.sensitivity=b(2);%slope - mV/mN
    Calibration.Rsquared=Rsq;
    Calibration.isvalid=true;
    Calibration.datetime=datestr(clock);
    Calibration.shunt=true;
    Calibration.masses=mass;
    Calibration.weights=Weight;
    Calibration.volts=Cal_Volts;
    Calibration.fit=b(1)+b(2)*Weight;

    save(['Calibration\',fname,'.mat'],'Calibration');
end
